from __future__ import annotations

from pydantic import BaseModel, ConfigDict, Field


class Driver(BaseModel):
    fullname: str = Field(alias="fullName")


class OsagoModel(BaseModel):
    model_config = ConfigDict(extra="ignore")
        
    masked_phone: str = Field(alias="maskedPhone")
    username: str = Field(alias="userName")
    brand_name: str = Field(alias="brandName")
    model_name: str = Field(alias="modelName")
    order_hash: str = Field(alias="orderHash")
    company_id: int = Field(alias="companyId")
    product_id: int = Field(alias="productId")
    company_name: str = Field(alias="companyName")
    price: int
    previous_policy_number: str = Field(alias="previousPolicyNumber")
    vehicle_year: int = Field(alias="vehicleYear")
    policyend_date: str = Field(alias="policyEndDate")
    drivers: list[Driver]
    drivers_amount: int = Field(alias="driversAmount")
    car_number: str = Field(alias="carNumber")



class CarNotFound(BaseException):
    __module__ = object.__module__
    
    def __init__(self):
        super().__init__("I doesn't found your car...")


# class OsagoNotFound(BaseException):
#     __module__ = object.__module__
    
#     def __init__(self):
#         super().__init__("Не удалось найти данные ОСАГО")


from abc import ABC, abstractmethod
from typing import Any, Generic, Literal, TypeVar

from bs4 import BeautifulSoup
from httpx import Response
from pydantic import __version__ as pydantic_version

from get_car_info.models import CarSnapshotModel, OsagoModel

ModeLiteral = Literal["gosnumber", "vin"]


T = TypeVar("T", bound="BaseCarInfo")
class BaseCarInfo(ABC):
    class Auth(ABC, Generic[T]):
        def __init__(self, car_info: T):
            self.car_info = car_info

        @abstractmethod
        def _get_auth_data(self, obj: str, mode: ModeLiteral) -> Response:
            ...

        @staticmethod
        def _parse_auth_data(response: Response) -> tuple:
            soup = BeautifulSoup(response.content, "html.parser")
            token: str = soup.find("input", {"name": "_token"})["value"].strip() #type: ignore
            snapshot: str = soup.find("div", {"x-init": "$wire.getDetails()"})["wire:snapshot"] # type: ignore
            return (token, snapshot)

    class API(ABC, Generic[T]):
        def __init__(self, car_info: T):
            self.car_info = car_info

        @abstractmethod
        def _get_result(self, obj: str, mode: ModeLiteral) -> Response:
            ...

        @staticmethod
        def _get_json_data(token: str, snapshot: dict) -> dict:
            return {
                "_token": token,
                "components": [
                    {
                        "snapshot": snapshot,
                        "updates": {},
                        "calls": [
                            {
                                "path": "",
                                "method": "getDetails",
                                "params": []
                            }
                        ]
                    }
                ]
            }

    def __init__(self):
        self.auth = self.Auth(self) # type: ignore
        self.api = self.API(self) # type: ignore

    @staticmethod
    def _get_model[Model](model: Model, data: Any) -> Model:
        if pydantic_version.split(".")[0] == "1":
            return model.parse_obj(data) # type: ignore
        
        if pydantic_version.split(".")[0] == "2":
            return model.model_validate(data) # type: ignore
        
        raise ValueError("support pydantic version not found")
    
    @abstractmethod
    def get_data(self, car_number: str) -> CarSnapshotModel:
        ...
    
    
    @abstractmethod
    def get_osago(self) -> OsagoModel | None:
        ...


import json

from httpx import AsyncClient, Client

from ..base import BaseCarInfo, ModeLiteral
from ..exceptions import CarNotFound
from ..models import CarSnapshotModel, OsagoModel


class CarInfo(BaseCarInfo):
    class Auth(BaseCarInfo.Auth["CarInfo"]):
        def _get_auth_data(self, obj: str, mode: ModeLiteral = "gosnumber") -> tuple:
            
            res = self.car_info._client.get(f"order/create?object={obj}&mode={mode}")
            return self._parse_auth_data(res)

    class API(BaseCarInfo.API["CarInfo"]):
        def _get_result(self, obj: str, mode: ModeLiteral) -> dict:
            token, snapshot = self.car_info.auth._get_auth_data(obj, mode) # type: ignore
            res = self.car_info._client.post("livewire/update", json=self._get_json_data(token, snapshot))
            res = res.json().get("components")[0].get("snapshot")
            return json.loads(res)

    def __init__(self) -> None:
        super().__init__()
        self._client = Client(base_url="https://vinvision.ru/")

    def get_data(self, *, car_number: str | None = None, vin: str | None = None) -> type[CarSnapshotModel]:
        error_msg = "Должен быть указан или гос номер, или vin автомобиля"
        assert any((car_number, vin)), error_msg
        assert not all((car_number, vin)), error_msg
        
        if car_number:
            mode = "gosnumber"
            
        elif vin:
            mode = "vin"
        
        obj: str = (car_number or vin)   # type: ignore
        res = self.api._get_result(obj=obj, mode=mode)

        try:
            first_result = res.get("data").get("details")[0].get("result")[0] # type: ignore
        except Exception:
            raise CarNotFound from None
        res = {i.lower(): k for i, k in first_result.items()}

        return self._get_model(CarSnapshotModel, res)
    
    def get_osago(self, car_number: str | type[CarSnapshotModel]) -> type[OsagoModel] | None:
        """ Попытка получения данных ОСАГО """
        
        if not isinstance(car_number, str):
            car_number = car_number.number
        
        url = "https://www.sravni.ru/proxy-osagoinsurance/getPrevCalculationOrPolicy/"
        data = {"carNumber": car_number.upper(), "isShortProlongation": True}
        
        response = Client().post(url=url, data=data).json()
        
        if len(response.keys()) == 1:
            return None
        
        return self._get_model(OsagoModel, response)
        


class AsyncCarInfo(BaseCarInfo):
    class Auth(BaseCarInfo.Auth["AsyncCarInfo"]):
        async def _get_auth_data(self, obj: str, mode: ModeLiteral = "gosnumber") -> tuple:
            
            res = await self.car_info._client.get(f"order/create?object={obj}&mode={mode}")
            return self._parse_auth_data(res)

    class API(BaseCarInfo.API["AsyncCarInfo"]):
        async def _get_result(self, obj: str, mode: ModeLiteral) -> dict:
            token, snapshot = await self.car_info.auth._get_auth_data(obj, mode) # type: ignore
            res = await self.car_info._client.post("livewire/update", json=self._get_json_data(token, snapshot))
            res = res.json().get("components")[0].get("snapshot")
            return json.loads(res)

    def __init__(self) -> None:
        super().__init__()
        self._client = AsyncClient(base_url="https://vinvision.ru/")

    async def get_data(self, *, car_number: str | None = None, vin: str | None = None) -> type[CarSnapshotModel]:
        error_msg = "Должен быть указан или гос номер, или vin автомобиля"
        assert any((car_number, vin)), error_msg
        assert not all((car_number, vin)), error_msg
        
        if car_number:
            mode = "gosnumber"
            
        elif vin:
            mode = "vin"
        
        obj: str = (car_number or vin)   # type: ignore
        res = await self.api._get_result(obj=obj, mode=mode) # type: ignore

        try:
            first_result = res.get("data").get("details")[0].get("result")[0]
        except Exception:
            raise CarNotFound from None
        res = {i.lower(): k for i, k in first_result.items()}

        return self._get_model(CarSnapshotModel, res)
    
    async def get_osago(self, car_number: str | type[CarSnapshotModel]) -> type[OsagoModel] | None:
        """ Попытка получения данных ОСАГО """
        
        if not isinstance(car_number, str):
            car_number = car_number.number
        
        url = "https://www.sravni.ru/proxy-osagoinsurance/getPrevCalculationOrPolicy/"
        data = {"carNumber": car_number.upper(), "isShortProlongation": True}
        
        response = await AsyncClient().post(url=url, data=data)
        response = response.json()
        
        if len(response.keys()) == 1:
            return None
        
        return self._get_model(OsagoModel, response)


import asyncio

from get_car_info import AsyncCarInfo


async def main() -> None:
    car_info = AsyncCarInfo()
    number = "а123вв15"

    model = await car_info.get_data(car_number=number)
    print(model)
    
    osago = await car_info.get_osago(car_number=number)
    
    if osago is None:
        print("ОСАГО не найдено.")
    else:
        print(osago)
    

if __name__ == "__main__":
    asyncio.run(main())



from pydantic import BaseModel


class CarSnapshotModel(BaseModel):
    vin: str
    number: str
    marka: str | None
    model: str | None
    year: int
    color: str | None
    volume: int | float
    horsepower: int | float
    image: str | None
    